prompts = {
    "comment": "Add or update comments according to the given language's standards. Add or update function, module, or class level comments if they're appropriate.",
    "varnames": "Change variable names, but nothing else, to make the code more readable. For example, instead of using 'x' and 'y', use 'width' and 'height'.",
    "ugh": "Do anything you can to make this code more readable. Add comments, change variable and function names, add whitespace, whatever. Add a readme to explain what the code does and where it could be improved.",
    "docs": "Generate documentation in markdown format for the given files. Make sure to include a README.md for github. If provided markdown files, update them and use their current structure.",
    "bugs": """Find any bugs you can, note them in comments prefixed with BUG:

Before:
def divide(a, b):
    return a / b

After:
def divide(a, b):
    # BUG: This function should check for division by zero.
    return a / b
""",
    "vulns": """Find any vulnerabilities you can, note them in comments prefixed with VULN:

Before:
def set_username(username):
    this.username = username

After:
def set_username(username):
    # VULN: This function should validate the input to prevent injection attacks.
    this.username = username
""",
}



import os
import typer
import json

from codegpt import prompts

from codegpt import gpt_interface as gpt
from codegpt import files

from typing import List
from pathlib import Path

app = typer.Typer(
    no_args_is_help=True,
)

app = typer.Typer()

@app.command("do")
def edit_file(
    instruction: str = typer.Argument(
        ..., help="Instruction to edit the file(s). Keep it short! Wrap with quotes.",
    ),
    filenames: List[Path] = typer.Argument(
        [], help="List of filenames to edit. If not provided, will prompt for input.",
    ),
    backup: bool = typer.Option(
        False, "--backup", "-b", help="Whether to create a backup of the original file(s).",
    ),
    yes: bool = typer.Option(False, "--yes", "-y", help="Don't ask for confirmation.",),
    raw_code: str = typer.Option(None, "--raw-code", "-c", help="Raw code to edit. Overrides filenames"),
    json_out: bool = typer.Option(False, "--json-out", "-j", help="Output to raw json."),
    ):
    """
    Do something given some code for context. Asking for documents, queries, etc. should work okay. Edits are iffy, but work a lot of the time.
    
    Your code better be in git before you use this.

    FILENAMES: list of filenames to edit. If not provided, will prompt for input.
    INSTRUCTION: the instruction to edit the file(s). Keep it short!
    """

    if not filenames and not raw_code:
        raise typer.BadParameter("Either FILENAMES or --raw-code (-c) must be provided.")
    code = {"code": raw_code} if raw_code else files.load_text(filenames)
    result = gpt.send_iffy_edit(instruction, code, yes=yes, clipboard=bool(raw_code))

    if json_out:
        return json.dumps(result, sort_keys=True, indent=4)

    files.write_text(result, backup)
    typer.secho("Done!", color=typer.colors.BRIGHT_BLUE)


@app.command("quick")
def quick_edit_file(
    option: str = typer.Argument(..., help=f"{{{'|'.join(prompts.prompts.keys())}}}"),
    filenames: List[str] = typer.Argument(..., help="Enter the filenames to edit, separated by spaces"),
        backup: bool = typer.Option(
        False, "--backup", "-b", help="Whether to create a backup of the original file(s).",
    ),
    yes: bool = typer.Option(False, "--yes", "-y", help="Don't ask for confirmation.",),
    raw_code: str = typer.Option(None, "--raw-code", "-c", help="Raw code to edit. Overrides filenames"),
    json_out: bool = typer.Option(False, "--json", "-j", help="Output in JSON format"),
    ):
    """
    Edit a file using codegpt's built in prompts.

    Arguments for `option`:
    - comment - Adds or updates comments
    - varnames - Makes variable names reasonable
    - ugh - Do anything GPT can to make the code suck less (might break stuff...)
    - docs - Generate (or update) docs, including README.md
    - bugs - Comment in code where the bugs are if GPT sees them (iffy)
    - vulns - Comment in code where the vulns are if GPT sees them (iffy)
    """
    if option not in prompts.prompts:
        raise typer.BadParameter(f"{option} is not a valid option. Must be one of {list(prompts.prompts.keys())}")

    if not filenames and not raw_code:
        raise typer.BadParameter("Either FILENAMES or --raw-code (-c) must be provided.")

    code = {"code": raw_code} if raw_code else files.load_text(filenames)
    result = gpt.send_iffy_edit(prompts.prompts[option], code, yes=yes, clipboard=bool(raw_code))

    if json_out:
        return json.dumps(result, sort_keys=True, indent=4)

    files.write_text(result, backup)
    typer.secho("Done!", color=typer.colors.BRIGHT_BLUE)


@app.command("config")
def config():
    """
    Configuration instructions for the OpenAI secret key for the codegpt CLI.
    """
    # check if the secret key is already set in the environment variables
    if "OPENAI_SECRET_KEY" in os.environ:
        typer.secho("OPENAI_SECRET_KEY is already set in the environment! You probably don't need this.", typer.colors.BRIGHT_BLUE)
    else:
        typer.confirm(
            """
I recommend setting your API key as an environment variable:
https://help.openai.com/en/articles/5112595-best-practices-for-api-key-safety

Windows users can also use `setx` like:

`$ setx OPENAI_SECRET_KEY=<YOUR_API_KEY>`

from an admin console.""".strip()
        )


if __name__ == "__main__":
    app()



import os
import typer

def load_text(filenames):
    out = {}
    for filename in filenames:
        with open(filename, "r") as f:
            out[filename] = f.read()
    return out


def write_text(files, backup=False):
    # If the backup option is specified and the file exists,
    # write the existing file to <filename>.bak
    for i, out in enumerate(files):
        filename = out.get('filename', f"{i}.txt")
        typer.secho(f"Hmm, didn't find a filename, writing to {filename}", color=typer.colors.MAGENTA)
        if backup and os.path.exists(filename):
            with open(filename, "r") as f_in:
                with open(f"{filename}.bak", "w") as f_out:
                    f_out.write(f_in.read())

        # Write the new text to the file
        with open(filename, "w") as f:
            f.write(out['code'])

        typer.secho(f"{filename} - " + out['explanation'], color=typer.colors.BLUE)



import nltk
import openai
import typer
from typing import Dict
from textwrap import dedent
from codegpt.parse import parse_resp

try:
    nltk.data.find('tokenizers/punkt')
except LookupError:
    typer.secho("Downloading punkt for nltk... Only once!", fg=typer.colors.GREEN, bold=True)
    nltk.download('punkt', quiet=True)

def confirm_send(prompt, max_tokens=4000, yes=False):
    tokens = nltk.word_tokenize(prompt)

    #! Yeah this math is BS, closeish though...
    max_tokens = round(max_tokens - (7 / 4) * len(tokens))

    if yes:
        typer.secho(f"This prompt is {len(tokens)}ish tokens, GPT-3 can return {max_tokens}ish.", color=typer.colors.GREEN)
    else:
        typer.confirm(
            f"This prompt is {len(tokens)}ish tokens, are you sure you want to continue?\nThe most GPT-3 can return in response is {max_tokens}ish.",
            default=True,
            abort=True,
        )

    return max_tokens

def send_iffy_edit(prompt: str, code: Dict[str, str], clipboard: bool = False, yes=False) -> str:
    full_prompt = "You are an expert developer. Given the following code:\n\n"

    for filename, code_string in code.items():
        if not clipboard:
            full_prompt += f"\n\nfilename:\n> {filename}\n"
        nl = '\n'
        full_prompt += f"code:\n{nl.join(f'> {x}' for x in code_string.splitlines())}\n\n"

    full_prompt += f"\n\nDo the following: {prompt}"

    if clipboard:
        full_prompt += dedent("""
        
        Answer in the following format:
        
        explanation:
        > <The changes that you made>
        code:
        > <the code to be output line 1>
        > <the code to be output, line n...>""")

    else:
        full_prompt += dedent("""

        You may only output complete files.

        If you add or modify a file, return it in this exact format:

        filename:
        > <the filename to be output>
        explanation:
        > <The changes that you made>
        code:
        > <code line 1>
        > <code line n...>""")

    max_tokens = confirm_send(full_prompt, yes=yes)

    response = openai.Completion.create(
        engine="text-davinci-003",
        prompt=full_prompt,
        max_tokens=max_tokens,
        n=1,
        temperature=0.5,
    )
    
    parsed = parse_resp(response)
    
    return parsed[0] if clipboard else parsed



if __name__ == "__main__":
    print(send_iffy_edit("Remove usless imports", {'filename': 'hello.py', 'code': """import os
import json
import sys
print("Hello, world!")"""}))



def parse_resp(response:dict):
    resp = response["choices"][0]["text"].splitlines()
    
    # Initialize an empty list to hold the dictionaries
    out = []

    # Initialize an empty dictionary to hold the current key-value pairs
    curr_dict = {}

    # Iterate through each line in the response
    for line in resp:
        line = line.strip()
        # Skip empty lines
        if not line:
            continue

        # If the line doesn't start with '>', it's a key
        if line[0] != '>':
            if line == '===' and curr_dict:
                out.append(curr_dict)
                curr_dict = {}
            # Strip leading/trailing whitespace and remove ':' from the key
            key = line.strip().replace(":", '')
            # Initialize an empty value for this key in the current dictionary
            curr_dict[key] = ""
        # If the line does start with '>', it's a value
        else:
            # Strip the leading '>' and leading/trailing whitespace from the value
            # and add it to the current key in the current dictionary
            if key == "code":
                curr_dict[key] += line.strip().strip('> ').strip('>') + '\n'
            else:
                curr_dict[key] = line.strip().strip('> ').strip('>').strip()

    # Add the final dictionary to the output list
    out.append(curr_dict)

    return out


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['codegpt']

package_data = \
{'': ['*']}

install_requires = \
['nltk>=3.7,<4.0', 'openai>=0.2,<0.3', 'typer>=0.4.0,<0.5.0']

entry_points = \
{'console_scripts': ['codegpt = codegpt.main:app']}

setup_kwargs = {
    'name': 'codegpt',
    'version': '0.2.10',
    'description': "A CLI tool for refactoring code using OpenAI's models",
    'long_description': '# Codegpt\n\n## 0.2.10\n\nA tool for using GPT just a little quicker. A nearly truly automated footgun. Learn how to revert with git before trying please.\n\nPosting about progress here:\n\n[![Twitter Follow](https://img.shields.io/twitter/follow/_JohnPartee?style=social)](https://twitter.com/_JohnPartee)\n\n## Getting Started\n\n`pip install codegpt --upgrade`\n\nAnd set your openapi API key as an environment variable like they recommend:\n[In their docs here](https://help.openai.com/en/articles/5112595-best-practices-for-api-key-safety)\n\nWindows users can also use `setx` like:\n\n`$ setx OPENAI_SECRET_KEY=<YOUR_API_KEY>`\n\nfrom an admin console.\n\n## Be careful! But try this\n\nUsage\nTo try Codegpt, you can run the following command:\n\n```bash\ncodegpt do <instructions (quoted)> <filenames>\n```\n\nIt will prompt you for directions to follow, and it\'ll do whatever you want. Write new docs, add comments to code, just make sure to ask it to "edit" or "comment" or whatever so it knows to change files or not.\n\nOr use the quick command to do some neat stuff, like:\n\nGenerate docs\n\n```bash\ncodegpt quick docs <filenames>\n```\n\nChange variable names to be more readable\n\n```bash\ncodegpt quick varnames <filenames>\n```\n\nAdd comments to your code automatically\n\n```bash\ncodegpt quick comment <filenames>\n```\n\nCheck for bugs (iffy, but worth a shot)\n\n```bash\ncodegpt quick bugs <filenames>\n```\n\nCheck for vulnerabilities (even more iffy, but worth a shot)\n\n```bash\ncodegpt quick vulns <filenames>\n```\n\nTry to make code less miserable\n\n```bash\ncodegpt quick ugh <filenames>\n```\n\nKeep in mind that using GPT-3 for code generation is paid, with a cost of 2 cents per 1,000 tokens.\n\nJust like with a Jr Dev, it\'s best to break up your tasks into smaller pieces to improve the results.\n',
    'author': 'John Partee',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/morganpartee/codegpt',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
